// Copyright Project Harbor Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package create

import (
	"context"
	"errors"
	"fmt"
	"strings"

	"github.com/charmbracelet/huh"
	"github.com/goharbor/go-client/pkg/sdk/v2.0/client/registry"
	"github.com/goharbor/harbor-cli/pkg/utils"
	"github.com/spf13/viper"
)

type CreateView struct {
	ProjectName  string
	Public       bool
	RegistryID   string
	StorageLimit string
	ProxyCache   bool
}

func getRegistryList() (*registry.ListRegistriesOK, error) {
	credentialName := viper.GetString("current-credential-name")
	client, err := utils.GetClientByCredentialName(credentialName)
	if err != nil {
		return nil, err
	}
	ctx := context.Background()
	var response *registry.ListRegistriesOK
	response, err = client.Registry.ListRegistries(ctx, &registry.ListRegistriesParams{})
	if err != nil {
		return nil, err
	}

	return response, nil
}

func CreateProjectView(createView *CreateView) error {
	theme := huh.ThemeCharm()
	// I want it to be a map of registry ID to registry name
	registries, err := getRegistryList()
	if err != nil {
		return err
	}
	registryOptions := map[string]string{}
	for _, registry := range registries.Payload {
		regiId := fmt.Sprintf("%d", registry.ID)
		registryOptions[regiId] = fmt.Sprintf("%s (%s)", registry.Name, registry.URL)
	}

	var registrySelectOptions []huh.Option[string]
	for id, name := range registryOptions {
		registrySelectOptions = append(registrySelectOptions, huh.NewOption(name, id))
	}

	err = huh.NewForm(
		huh.NewGroup(
			huh.NewInput().
				Title("Project Name").
				Value(&createView.ProjectName).
				Validate(func(str string) error {
					if strings.TrimSpace(str) == "" {
						return errors.New("project name cannot be empty or only spaces")
					}
					if isValid := utils.ValidateProjectName(str); !isValid {
						return errors.New("please enter correct project name format")
					}
					return nil
				}),
			huh.NewConfirm().
				Title("Public").
				Value(&createView.Public).
				Affirmative("yes").
				Negative("no"),
			huh.NewInput().
				Title("Storage Limit").
				Value(&createView.StorageLimit).
				Validate(func(str string) error {
					// Assuming StorageLimit is an int64
					if strings.TrimSpace(str) == "" {
						return errors.New("storage limit cannot be empty or only spaces")
					}
					if err := utils.ValidateStorageLimit(str); err != nil {
						return err
					}
					return nil
				}),

			huh.NewConfirm().
				Title("Proxy Cache").
				Value(&createView.ProxyCache).
				Affirmative("yes").
				Negative("no"),
		),
		huh.NewGroup(
			huh.NewSelect[string]().
				Validate(func(str string) error {
					if createView.ProxyCache && str == "" {
						return errors.New("registry ID cannot be empty")
					}
					return nil
				}).
				Description("Select a registry to reference when creating the proxy cache project").
				Title("Registry ID").
				Value(&createView.RegistryID).
				Options(registrySelectOptions...),
		).WithHideFunc(func() bool {
			return !createView.ProxyCache || len(registryOptions) == 0
		}),
	).WithTheme(theme).Run()

	if err != nil {
		return err
	}
	return nil
}
